<?php

namespace Drupal\standwithukraine\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure StandWithUkraine favicon settings for this site.
 */
class StandWithUkraineFaviconForm extends StandWithUkraineFormBase {

  const TYPES = [FALSE => 'foreground', TRUE => 'background'];

  /**
   * The favicon.
   *
   * @var \Drupal\standwithukraine\Service\StandWithUkraineFaviconInterface
   */
  protected $favicon;

  /**
   * The cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $invalidator;

  /**
   * The time.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->favicon = $container->get('standwithukraine.favicon');
    $instance->invalidator = $container->get('cache_tags.invalidator');
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getName() {
    return 'favicon';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->config = $this->config($this->getEditableConfigNames()[0]);

    $form['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#description' => $this->t('Activate functionality for styling favicon.'),
      '#default_value' => !$this->read('disable'),
    ];

    $form['themes'] = [
      '#type' => 'details',
      '#title' => $this->t('Themes'),
      '#description' => $this->t('Configure favicon for default theme or admin
theme.'),
      '#tree' => TRUE,
      '#open' => FALSE,
      '#states' => [
        'disabled' => [
          ':input[name="favicon[enable]"]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $theme_config = $this->config('system.theme');
    $themes = (array) $this->read('themes');
    $suffix = '?timestamp=' . $this->time->getRequestTime();

    /** @var \Drupal\Core\StringTranslation\TranslatableMarkup $title */
    foreach ([$this->t('Default'), $this->t('Admin')] as $title) {
      $theme_type = strtolower($title->getUntranslatedString());
      $theme_name = $theme_config->get($theme_type);

      if (empty($theme_name)) {
        continue;
      }

      $favicon = $this->favicon->build($theme_name);

      if (!isset($favicon[$theme_name])) {
        continue;
      }

      if (isset($themes[$theme_name])) {
        $ratio = (int) $themes[$theme_name]['ratio'];
      }
      elseif (in_array($theme_name, StandWithUkraineFaviconInterface::THEMES)) {
        $ratio = StandWithUkraineFaviconInterface::DEFINED_RATIO;
      }
      else {
        $ratio = StandWithUkraineFaviconInterface::UNDEFINED_RATIO;
      }

      $custom = $ratio !== StandWithUkraineFaviconInterface::UNDEFINED_RATIO;

      $form['themes']['#open'] |= $custom;

      $form['themes'][$theme_name] = [
        '#type' => 'details',
        '#title' => $title,
        '#open' => $custom,
      ];

      $form['themes'][$theme_name]['preview'] = [
        '#theme' => 'table',
        '#caption' => $this->t('Preview'),
        '#attributes' => [
          'class' => ['preview'],
        ],
        '#attached' => [
          'library' => ['standwithukraine/form'],
        ],
      ];

      $rows = &$form['themes'][$theme_name]['preview']['#rows'];

      foreach ($favicon[$theme_name] as $dimensions => $url) {
        $rows['images'][]['data'] = [
          '#type' => 'html_tag',
          '#tag' => 'img',
          '#attributes' => [
            'src' => $url . $suffix,
          ],
        ];

        $rows['dimensions'][]['data'] = $dimensions;
      }

      if (($columns = count($rows['images'])) > 1) {
        for ($current_column = 0; $current_column < $columns - 1; $current_column++) {
          for ($next_column = $current_column + 1; $next_column < $columns; $next_column++) {
            $sizes = array_map(
              function ($column) use ($rows) {
                return explode('x', $rows['dimensions'][$column]['data'])[0];
              },
              [$current_column, $next_column],
            );

            if ($sizes[0] > $sizes[1]) {
              foreach (array_keys($rows) as $row) {
                $buffer = $rows[$row][$current_column];
                $rows[$row][$current_column] = $rows[$row][$next_column];
                $rows[$row][$next_column] = $buffer;
              }
            }
          }
        }
      }

      $form['themes'][$theme_name]['ratio'] = [
        '#type' => 'number',
        '#title' => $this->t('Ratio'),
        '#description' => $this->t('Define how many percentages of icon height
should be dedicated to blue color.'),
        '#min' => 10,
        '#max' => 90,
        '#default_value' => $ratio,
        '#field_suffix' => '%',
      ];

      $form['themes'][$theme_name]['type'] = [
        '#type' => 'radios',
        '#title' => $this->t('Type'),
        '#options' => [
          'foreground' => $this->t('Foreground'),
          'background' => $this->t('Background'),
        ],
        '#default_value' => self::TYPES[(bool) ($themes[$theme_name]['background'] ?? 0)],
        '#pre_render' => [
          [self::class, 'preRenderCompositeFormElement'],
          [self::class, 'preRenderField'],
        ],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    $config->set('favicon.disable', !$form_state->getValue('enable'));

    foreach (Element::children($form['themes']) as $theme_name) {
      $path = ['themes', $theme_name];
      $name = 'favicon.' . implode('.', $path) . '.';

      $config
        ->set(
          $name . 'ratio',
          $form_state->getValue(array_merge($path, ['ratio']))
        )
        ->set(
          $name . 'background',
          array_search(
            $form_state->getValue(array_merge($path, ['type'])),
            self::TYPES
          )
        );
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * Describe both types of favicon styling.
   */
  public static function preRenderField(array $element) {
    $element['foreground']['#description'] = t('Change colors of the visible
part of images and leave the transparent background as is.');

    $element['background']['#description'] = t('Add the Ukraine flag behind of
image visible part and leave the second one as is.');

    return $element;
  }

}
